/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// dribbleaction.cpp Mon Jun 11 2001
// Description:
//////////////////////////////////////////////////////////////////////

#include "dribbleaction.h"
#include "kickcommand.h"

bool DribbleAction::fTriedBackKick = false;
const double DribbleAction::DRIBBLE_POWER = 23;

DribbleAction::DribbleAction()
{
	setDuration(1);
}

DribbleAction::DribbleAction(const Angle &_aDirection)
{
	setDuration(1);
	setDirection(_aDirection);
}

DribbleAction::~DribbleAction()
{
}

Action* DribbleAction::copy() const
{
	return new DribbleAction(*this);
}

ServerCommand* DribbleAction::translate() const
{
	// WARNING: creating new object, remember to delete!
	// kick the ball in the desired direction... kick it to the side, if a back kick won't work
	KickCommand* pkcCmd;
	
	if ((getDirection() > (2.0 * PI_3)) || (getDirection() < (-2.0 * PI_3)) && DribbleAction::fTriedBackKick)
	{
		Angle aDir;
		
		if (getDirection() < 0)
			aDir = (-2.0 * PI_3);
		else
			aDir = (2.0 * PI_3);
		
		pkcCmd = new KickCommand(DribbleAction::DRIBBLE_POWER, aDir);
		DribbleAction::fTriedBackKick = false;
	}
	else if ((getDirection() > PI_2) || (getDirection() < -PI_2) && !DribbleAction::fTriedBackKick)
	{
		pkcCmd = new KickCommand(DribbleAction::DRIBBLE_POWER, getDirection());
		DribbleAction::fTriedBackKick = true;
	}
	else
	{
		pkcCmd = new KickCommand(DribbleAction::DRIBBLE_POWER, getDirection());
		DribbleAction::fTriedBackKick = false;
	}
	
	return pkcCmd;
}

void DribbleAction::setDirection (const Angle &_aDirection)
{
	aDirection = _aDirection;
}

Angle DribbleAction::getDirection () const
{
	return aDirection;
}

ostream& DribbleAction::ostream_insert(ostream& _out) const
{
    _out << "DribbleAction" << endl;

    return _out;
}
ofstream& DribbleAction::ofstream_insert(ofstream& _fout) const
{
    // TODO: Implement XML
    _fout << "<dribbleaction/>";

    return _fout;
}
