//////////////////////////////////////////////////////////////////////
// Mainz Rolling Brains RoboCup agents developed for the Simulator
// League of the RoboCup 2000 competition
//
// Copyright (C) 1998, 1999, 2000 Team Members
// of the Mainz Rolling Brains RoboCup Team
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//-*- c++ -*-/////////////////////////////////////////////////////////
//
//  $Id: comm.cpp,v 1.6 2001/07/23 21:18:00 jmm9gc Exp $
//  $Manager: eri$
//
//
//  comm.cpp - Communication with soccer server
//
//////////////////////////////////////////////////////////////////////

#include "comm.h"
#include "types.h"
#include "debug.h"

#undef TRACE
#define TRACE(cmd)

Comm::Comm() : LowLevelComm()
{

    TRACE("Comm: constructor entered" << endl);

    hadMessage = false;
    noMessageCount = 0;
		

    TRACE("Comm: constructor completed");

}


Comm::~Comm()
{

    TRACE("Comm: destructor entered");

    // log out
    if (isServerAlive())
        sendToServer ("(bye)");
/*
    if (parser)
        delete parser;
*/
/*
    if (store)
        delete store;
    if (audiomgr)
        delete audiomgr;
*/
    TRACE("Comm: destructor completed");

}

void Comm::finishAlarm()
{

    TRACE("Comm: handling SIGALRM");

    if (hadMessage)
    {
        noMessageCount = 0;
        hadMessage = false;
    }
    else
    {
        TRACE("Comm: no message from server since last alarm (for "
              << noMessageCount+1 << " steps now)");
        noMessageCount++;
    }

    if (noMessageCount > COMM_MAX_DEAD_SERVER_CYCLES)
    {
        TRACE("Comm: no messages for more than "
              << COMM_MAX_DEAD_SERVER_CYCLES
              << " supposed timesteps, server is dead.");
        serverOk = false;
    }

}

void Comm::finishIo(SensorDataStore &_store)
{

    TRACE("Comm: handling SIGIO");
    char msg [MAX_SERVER_MSG_SIZE];

    while (receiveFromServer(msg, 0) == COMM_OK)
    {

        hadMessage = true;

        SensorData* data = paParser.parseServerString(msg);
        if (data == NULL)
        {
            TRACE("servertime_here"
                  << " Comm: could not parse server string " << msg);
        }
        else
        {

            _store.insert(data);
						// use data to adjust time, play mode etc.
            // bool firstSenseBody = info->adjust(data);

						// this switch should be on the proper section of the message string
						// and the create the proper type of sensordata object
						// the whole switch should probably be in another function called parse

						// to even make it work now, need to reformat with new sensordata types
						// (no audiomanager

/*
            switch (data.type)
            {

            case SENSOR_TYPE_OK:
                TRACE("Comm: skipping ok message");
                break;
            case SENSOR_TYPE_ERROR:
                TRACE("servertime_here"
                      << " Comm (" << info->getTeam() << " No." << info->getNumber()
                      << "): received error message from server: "
                      << msg );
                break;

            case SENSOR_TYPE_SENSE_BODY:
                //if (info->isTimeRunning() && firstSenseBody)
                //{
                    // first sense_body of a timestep received while clock is running:
                    // insert it before all data from same timestep
                    // (needed by worldmodel)
                    //TRACE("Comm: inserting first sense_body data into store");
                    //store->insert(data);
                //}
                //else
                //{
                    // additional sense_body, or sense_body received while clock
                    // is stopped: just store it according to receive time
                    TRACE("Comm: storing sense_body data");
                    store->put(data);
                //}
                break;
            case SENSOR_TYPE_EXACT:
                TRACE("Comm: storing exact data");
                store->put(data);
                break;
            case SENSOR_TYPE_SEE:
                TRACE("Comm: storing see data");
                store->put(data); // worldmodel can handle twisted neck
                break;
            case SENSOR_TYPE_HEAR:

                switch (data.sender)
                {
                case SENDER_SELF:
                    TRACE("Comm: skipping own message");
                    break;
                case SENDER_REFEREE:
                    TRACE("Comm: skipping referee play mode");
                    // information contained was already used in info->adjust() above,
                    // so also forget about it
                    break;
                case SENDER_DIR:    // message from player heard by player
                    TRACE("Comm: storing heard message");
                    TRACE("Comm: decoding via AudioMgr "
                          << data.message);
                    audiomgr->storeMsg(AudioMsg(data.message));
                    break;
                case SENDER_PLAYER: // message from player heard by any coach
                    TRACE("Comm: storing player message");
                    TRACE("Comm: decoding via AudioMgr "
                          << data.message);
                    audiomgr->storeMsg(AudioMsg(data.message));
                    break;
                case SENDER_COACH:  // message from train coach heard by player/olcoach
                    if (info->isTrainCoach())
                    {
                        TRACE("Comm: skipping own message");
                    }
                    else
                    {
                        TRACE("Comm: storing coach message relayed "
                              << "through the referee" );
                        TRACE("Comm: decoding via AudioMgr "
                              << data.message);
                        audiomgr->storeMsg(AudioMsg(data.message));
                    }
                    break;
                case SENDER_ONLINECOACH:// msg from onlinecoach heard by player/coach
                    if (info->isOnlineCoach())
                    {
                        TRACE("Comm: skipping own message");
                    }
                    else
                    {
                        TRACE("Comm: storing onlinecoach message");
                        TRACE("Comm: decoding via AudioMgr "
                              << data.message);
                        audiomgr->storeMsg(AudioMsg(data.message));
                    }
                    break;
                } // switch on sender

                break;

            } //switch on sensor_type
*/
        } //parse ok

    } // while data

}



// Override LowLevelComm::init with Comm::init to allow a checkin with the server
//	when the signals are turned off

comm_error Comm::init(IdentityData &_idAboutMe,
           const char* hostname,
           int port,
           int reconnectNumber)
{

		TRACE("Comm: initializing and checking in");

    comm_error e;

		e = blockSignals();
    if(e)
        return e;

    e = createSocket (hostname, port);
    if (e)
        return e;

    e = checkinAtServer (_idAboutMe, reconnectNumber);
    if (e)
        return e;

		// WARNING: Do something about this shit
		// set parameters known after receiving first info
		// audiomgr->setStates(aboutMe->number, aboutMe->side == SIDE_LEFT );
    paParser.setStates(_idAboutMe.team_name, _idAboutMe.number, _idAboutMe.side,
		                 (_idAboutMe.type == CLIENT_ONLINECOACH || _idAboutMe.type == CLIENT_TRAINCOACH));


    e = initSigHandlers();
    if (e)
        return e;

    e = setAlarmInterval(SIMULATOR_STEP);
    if (e)
        return e;

    e = unblockSignals();
    if(e)
        return e;

    serverOk = true;

    TRACE("Comm: init sucessfully completed");

    return COMM_OK;
}

// don't think this is necessary any longer b/c
// body will store sensor data's (it has to choose
// the most important ones)
/*
bool Comm::getNextData(SensorData &data)
{
    return store->getNext(data);
}
*/

// help not so sure about how to move actNow from AsyncCommInfo
// appears that this isn't really necessary, worldmodels can check
// timestamp of sensordata and return an error if same time
// handleSignals() won't do jack if no flags have come up
// could toggle variable from finishIo and getNextData to indicate
// if any new messages have arrived
bool Comm::isActionNeeded()
{
    //return info->actNow();
		return true;
}

// Won't get used... JM
//void Comm::say(AudioMsg &msg)
//{
// not yet implemented will probably look something like:
//	sendSay(msg.encode())
//    const char* charmsg = audiomgr->SpeakTo(msg);
//    TRACE("Comm: encoding via AudioMgr " << msg);
//    sendSay(charmsg);
//}

// don't think getNextMsg is necessary since
//	audiodata datatype exists and is on sensordata queue
/*
bool Comm::getNextMsg(AudioMsg &msg)
{
    bool hadMsg = audiomgr->getNextMsg(msg);
    if (hadMsg)
    {
        TRACE("Comm: retrieved from AudioMgr " << msg);
    }
    return hadMsg;
}
*/

/*
Probably won't get used since AudioMsg works just like an action
void Comm::sendSay(const char *message)
{

    unsigned int maxSize = MAX_SAY_MSG_SIZE;
// TODO: use PlayerComm/CoachComm to extend this function
//			and pass maxsize in to sendSay
//    if (info->isCoach())
//        maxSize = SAY_COACH_MSG_SIZE;

    // [!] wie lang darf duerfen die messages hoechstens sein (+-10 Zeichen?)
		// DOH: i don't speak german
    if (strlen(message)+10 > maxSize)
    {
        TRACE("servertime_here"
              << "Comm: client tries to send too long say message, dropping "
              << "it completely" );
        return;
    }

    char msg[maxSize];
    sprintf(msg, "(say %s)", message);

    if (sendToServer(msg) == COMM_OK)
    {
        TRACE("Comm: agent sent " << msg << endl);
    }
    else
    {
        TRACE("servertime_here"
              << " Comm: agent cannot send " << msg << " to server" );
    }
}
*/

comm_error Comm::checkinAtServer (IdentityData &_idAboutMe,int reconnectNumber)
{

   cout << "Comm:     ";
               switch(_idAboutMe.type)
           {
           case CLIENT_GOALIE:
               cout << "goalie ";
               break;
           case CLIENT_FIELDPLAYER:
               cout << "fieldplayer ";
               break;
           case CLIENT_ONLINECOACH:
               cout << "onlinecoach ";
               break;
           case CLIENT_TRAINCOACH:
               cout << "trainingcoach ";
               break;
           default:
               break;
           }
           if (reconnectNumber > 0) cout << "reconnecting" << endl;
               else
                   cout << "checking into server" << endl;


    char outmsg[MAX_SERVER_MSG_SIZE];
    char inmsg[MAX_SERVER_MSG_SIZE];

    generateCheckinMessage (outmsg, _idAboutMe, reconnectNumber);

    // send message to server
    TRACE ( "Comm:     sending checkin message "
               << outmsg << " to server" << endl);
    if (sendToServer(outmsg) != COMM_OK)
        return COMM_ERROR_SERVER_INIT;

    TRACE(  "Comm:     waiting for server response" << endl);

    if (receiveFromServer(inmsg, COMM_MAX_DEAD_SERVER_CYCLES*SIMULATOR_STEP)
            != COMM_OK)
        return COMM_ERROR_NO_DATA;
    TRACE ( "Comm:     received server response "
               << inmsg << endl);

    comm_error ce;
    ce = parseCheckinResponse (_idAboutMe, inmsg, reconnectNumber);
    if (ce != COMM_OK)
    {
        TRACE("Comm: cannot handle server checkin response " << inmsg );

        return ce;
    }

    TRACE(  "Comm:     completed checkin at sserver" << endl);

    return COMM_OK;

}

// todo: playercomm should override this to choose correct message
void Comm::generateCheckinMessage(char* outmsg,
                             const IdentityData &_idAboutMe, int reconnectNumber)
{

    if (reconnectNumber <= 0)
    { // player first init or any coach
        switch (_idAboutMe.type)
        {
        case CLIENT_GOALIE:
            sprintf(outmsg, "(init %s (version 6.00) (goalie))",
                    _idAboutMe.team_name);
            break;
        case CLIENT_FIELDPLAYER:
            sprintf(outmsg, "(init %s (version 6.00))", _idAboutMe.team_name);
            break;
        case CLIENT_ONLINECOACH:
            sprintf(outmsg, "(init %s (version 6.00))", _idAboutMe.team_name);
            break;
        case CLIENT_TRAINCOACH:
            sprintf(outmsg, "(eye on)");
            break;
        default:
            break;
        }
    }
    else
    {
        switch (_idAboutMe.type)
        {
        case CLIENT_GOALIE:
        case CLIENT_FIELDPLAYER:
            sprintf(outmsg, "(reconnect %s %d)",
                    _idAboutMe.team_name, reconnectNumber);
            break;
        case CLIENT_ONLINECOACH:
            ENSURE(reconnectNumber == 0); // coach cannot reconnect
        case CLIENT_TRAINCOACH:
            ENSURE(reconnectNumber == 0); // coach cannot reconnect
        default:
            break;
        }
    }

}

comm_error Comm::parseCheckinResponse(IdentityData &_idAboutMe, char* inmsg, int reconnectNumber)
{

    char   c;
    char   playmode [MAX_SERVER_MSG_SIZE];

    // needed to parse play mode returned by server
    Parser dummy("", 0, SIDE_UNKNOWN, false);

    int          time;
    bool         secondHalf;
    int          number = -1;
    play_mode    mode   = PLAY_MODE_UNKNOWN;
//    playing_side side   = SIDE_UNKNOWN;

    if (reconnectNumber <= 0)
    { // parse the init response
        time = 0;
        secondHalf = false;
        switch (_idAboutMe.type)
        {
        case CLIENT_GOALIE:
        case CLIENT_FIELDPLAYER:
            if (sscanf(inmsg, "(init %c %d %[^)])", &c, &number, playmode) != 3)
                return COMM_ERROR_PARSE;
            if (c == 'l')
                _idAboutMe.side = SIDE_LEFT;
            else if (c == 'r')
                _idAboutMe.side = SIDE_RIGHT;
            else
                return COMM_ERROR_PARSE;
            mode = dummy.parsePlayMode(playmode);
            break;
        // WARNING: why does this call sendEye???
				case CLIENT_ONLINECOACH:
            if (sscanf(inmsg, "(init %c ok)" , &c) != 1)
                return COMM_ERROR_PARSE;
            else
            {
                number = 0;
                if (c == 'l')
                    _idAboutMe.side = SIDE_LEFT;
                else if (c == 'r')
                    _idAboutMe.side = SIDE_RIGHT;
                else
                    return COMM_ERROR_PARSE;
                mode = PLAY_MODE_BEFORE_KICK_OFF;
                //sendEye(true);
            }
            break;
        case CLIENT_TRAINCOACH:
            if (strcmp(inmsg,"(ok eye on)"))
                return COMM_ERROR_PARSE;
            else
            {
                number = 0;
                // side = coachSide; <- what the hell were they thinking? OnlineCoach cannot change playmode (recall endEye starting game)
                if (_idAboutMe.side == SIDE_LEFT)
                    mode = PLAY_MODE_KICK_OFF_US;
                else
                    mode = PLAY_MODE_KICK_OFF_THEM;
                //sendEar(true);
            }
            break;
        default:
            break;
        }
    }
    else
    {  // parse the reconnect response
        time = 3000;
        secondHalf = true;
        switch (_idAboutMe.type)
        {
        case CLIENT_GOALIE:
        case CLIENT_FIELDPLAYER:
            if (sscanf(inmsg, "(reconnect %c %[^)])", &c, playmode) != 2)
                return COMM_ERROR_PARSE;
            number = reconnectNumber;
            if (c == 'l')
                _idAboutMe.side = SIDE_LEFT;
            else if (c == 'r')
                _idAboutMe.side = SIDE_RIGHT;
            else
                return COMM_ERROR_PARSE;
            mode = dummy.parsePlayMode(playmode);
            break;

        case CLIENT_ONLINECOACH:
            ENSURE(reconnectNumber == 0); // coach cannot reconnect
            break;

        case CLIENT_TRAINCOACH:
            ENSURE(reconnectNumber == 0); // coach cannot reconnect
            break;

        default:
            break;
        }
    }


	_idAboutMe.number = number; // Put in a better place later


    // playmode of game
    _idAboutMe.pmMode = mode; // Unknown if not set


// WARNING: This is where you learn your side of the field, and your player number
// was once:     info->setStates(type, number, time, mode, side, secondHalf);

    return COMM_OK;

}

