/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// vector2d.h Mon Jun 18 2001
// Description:
//////////////////////////////////////////////////////////////////////

#ifndef VECTOR2D_H
#define VECTOR2D_H

#include <iostream.h>
#include <math.h>
#include "constants.h"
#include "minmax.h"

#include "angle.h"
class Angle;

// Class for 2-dimensional coordinates (x,y).
// Zero angle is in direction of positive x-axis.

class Vector2d
{
public:

    Vector2d();
    Vector2d( double _x, double _y );

    double getX() const;

    double getY() const;

    double getLength() const;

    Angle getAngle() const;

    int operator == ( const Vector2d &c ) const;
    bool operator != (const Vector2d &c ) const;
		bool operator>(const Vector2d &c) const;
		bool operator<(const Vector2d &c) const;
		bool operator>=(const Vector2d &c) const;
		bool operator<=(const Vector2d &c) const;
		bool isApprox( const Vector2d &c ) const;
				
    void setX( double _x );

    void setY( double _y );


    void setXY( double _x, double _y );

    // init vector with polar coordinates
    void setRadAng( double r, Angle angle );

    // arithmetic (+,-)

    Vector2d operator - ( ) const;

    Vector2d &operator += ( const Vector2d &c );


    Vector2d &operator -= ( const Vector2d &c );


    Vector2d operator + ( const Vector2d &c ) const;


    Vector2d operator - ( const Vector2d &c ) const;


    // stretch vector by double value s (same as method 'stretch')
    Vector2d &operator *= ( double s );
    Vector2d &operator /= ( double s );

    Vector2d &stretch( double s );


    Vector2d operator * ( double s ) const;

    Vector2d operator / ( double s ) const;


    // just for writing v1 + 2 * v2 instead of v1 + v2 * 2
    friend Vector2d operator * ( double s, const Vector2d &c );


    // dot product
    double operator * ( const Vector2d &b ) const;

    // turn by angle alpha (uses sin and cos)
    Vector2d &turn( Angle alpha );

    // (faster) turn by +90 DEG
    Vector2d &perp();

private:
		static const double THRESHHOLD;
    double x, y;
};


// Intersect circle with line or circle or other line
enum Intersect { None, Valid, Instable, R1larger, R2larger };

Intersect intersectLineCircle( const Vector2d &linedirection,
                               const Vector2d &circlecenter,
                               double radius, double &l1, double &l2 );

Intersect intersectCircleCircle( double r1, const Vector2d &center2,
                                 double r2, Vector2d &l1, Vector2d &l2 );

Intersect intersectLineLine( const Vector2d &line1dir,
                             const Vector2d &line2offset,
                             const Vector2d &line2dir,
                             double &l1, double &l2);



// Utility functions
// Square of Euclidean norm
double sqrnorm( const Vector2d &c );

// Euclidean norm
double norm( const Vector2d &c );

// Distance between two points (vectors)
double dist( const Vector2d &a, const Vector2d &b );

// Generate Vector2d with polar coordinates
Vector2d radAng2Vector2d( double r, Angle angle );

// Vector output to stream
ostream& operator << (ostream &os, const Vector2d &c );


#endif
