/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// behindballperception.cpp Fri Jul 13 2001
// Description:
//////////////////////////////////////////////////////////////////////

#include "behindballperception.h"

BehindBallPerception::BehindBallPerception(const WorldModel &_wmWorld): Perception(_wmWorld)
{
}

BehindBallPerception::~BehindBallPerception()
{
}

Vector2d BehindBallPerception::getBallAbsPos()
{
//returns the absolute position of the ball
return wmWorld.getWorldBall().getAbsPosition();
}

bool BehindBallPerception::inAttackingThird(WorldObject _woObj)
{
	//gets the absolute position of the WMO
	Vector2d vAbsPos = _woObj.getAbsPosition();
	
	//finds if the X coord is greater than 1/6 Length because greater than that
	//is the attacking third of the field
	if ( vAbsPos.getX() > (FIELD_LENGTH / 6.0) )
		return true;
	else return false;}

bool BehindBallPerception::inMiddleThird(WorldObject _woObj)
{
	//gets the absolute position of the WMO
	Vector2d vAbsPos = _woObj.getAbsPosition();
	
	//finds if the X coord is less than 1/6 Length and greater than -1/6 Length
	//because that is the middle third of the field
	if ( (vAbsPos.getX() < (FIELD_LENGTH / 6.0)) && (vAbsPos.getX() > -(FIELD_LENGTH / 6.0)) )
		return true;
	else return false;}

bool BehindBallPerception::inDefensiveThird(WorldObject _woObj)
{
	//gets the absolute position of the WMO
	Vector2d vAbsPos = _woObj.getAbsPosition();
	
	//finds if the X coord is less than -1/6 Length because less than that
	//is the defensive third of the field
	if ( vAbsPos.getX() < -(FIELD_LENGTH / 6.0) )
		return true;
	else return false;
}

bool BehindBallPerception::inOurPenaltyArea(WorldObject _woObj)
{
  Vector2d vAbsPos = _woObj.getAbsPosition();

	return ( ( vAbsPos.getX() > - 0.5 * FIELD_LENGTH ) &&
	   ( vAbsPos.getX() < - 0.5 * FIELD_LENGTH + PENALTY_AREA_LENGTH ) &&
	   ( fabs( vAbsPos.getY() ) < 0.5 * PENALTY_AREA_WIDTH ) );
}

WorldBall BehindBallPerception::getBall()
{
	//returns the world ball
	return wmWorld.getWorldBall();
}

bool BehindBallPerception::isWellBehindBall ()
{

	if ( distToLine() > 0.0)
		return false;
	else
		return true;	
}

Angle BehindBallPerception::getBodyDir()
{
 	return ( wmWorld.getClientPlayer().getBodyDir() );
}


double BehindBallPerception::distToLine () const
{
  
  Vector2d vPos = wmWorld.getClientPlayer().getAbsPosition();
  double myX = vPos.getX();
  
  double ballX = wmWorld.getWorldBall().getAbsPosition().getX();

  
  if(inGoalBox(vPos)) // this way wellbehindball will be satisfied
    return -1.0;

  if (ballX < - 0.5 * FIELD_LENGTH + 5.0)
    return (myX + 0.5 * FIELD_LENGTH - 5.0); 

  else if (ballX < - 0.5 * FIELD_LENGTH + 20.0)
    return  (myX - ballX + 5.0);

  else if (xDistIndex() <= 3)
    return  (myX - ballX + 20.0);

  else
    return  (myX - ballX + 10.0);
  
}

int BehindBallPerception::xDistIndex() const
{
	int iClientNum = wmWorld.getClientPlayer().getNumber();

	//finds out the number of teammates client knows well enough for disperse
	int iArray = 0;
	for (int h = 1; h <= MAX_PLAYER; ++h)
		{
		if ( wmWorld.getOurPlayer(h).isYoung() )
			++iArray;
		}

	// create array for X distances
	WorldPlayer wpTeammateArray[iArray];
  WorldPlayer temp;
	
  //sets up teammate array
  int iArrayCount = 0;
	for (int k = 1; k <= MAX_PLAYER; ++k)
	{
		if ( wmWorld.getOurPlayer(k).isYoung() )
		{
			wpTeammateArray[iArrayCount] = wmWorld.getOurPlayer(k);
			++iArrayCount;
		}
	}

	// get X distance and resort the distance array
	for (int i = 0; i < iArray; ++i)
	{
		for (int j = 0; j < i; ++j)
		{
			if ( wpTeammateArray[j].getAbsPosition().getX()  >  wpTeammateArray[i].getAbsPosition().getX() )
			{
				temp = wpTeammateArray [i];
				wpTeammateArray [i] = wpTeammateArray [j];
				wpTeammateArray [j] = temp;
			}
		}
  }

	for (int g = 0; g < iArray; ++g)
	{
		if (wpTeammateArray[g].getNumber() == iClientNum)
			return (g + 1);
	}
	
	return 11;

}

int BehindBallPerception::getBallAge() const
{
	return ( wmWorld.getWorldBall().getAge() );
}

bool BehindBallPerception::isActiveMode() const
{
    play_mode pmTemp = wmWorld.getPlayMode();
    if( (pmTemp == PLAY_MODE_PLAY_ON) 
        || (pmTemp == PLAY_MODE_DROP_BALL)
        || (pmTemp == PLAY_MODE_KICK_IN_THEM)
        || (pmTemp == PLAY_MODE_KICK_IN_US)
        || (pmTemp == PLAY_MODE_FREE_KICK_THEM)
        || (pmTemp == PLAY_MODE_CORNER_KICK_THEM)
	|| (pmTemp == PLAY_MODE_GOAL_KICK_THEM)
      ) return true;

    return false;
}

bool BehindBallPerception::slowRun() const
{
  if(lowStamina())
    return true;

  play_mode pmTemp = wmWorld.getPlayMode();
    if( (pmTemp == PLAY_MODE_KICK_IN_US) ||
        (pmTemp == PLAY_MODE_KICK_IN_THEM) ||
        (pmTemp == PLAY_MODE_FREE_KICK_US) ||
        (pmTemp == PLAY_MODE_FREE_KICK_THEM)
      ) return true;

    return false;
}


bool BehindBallPerception::lowStamina() const
{
  return getStamina() < 1500;
}

bool BehindBallPerception::ballTooDeep() const
{
  return wmWorld.getWorldBall().getAbsPosition().getX() < (-FIELD_LENGTH/2.0 + 6);
}


bool BehindBallPerception::inGoalBox(Vector2d v) const
{
  if((v.getX() < -FIELD_LENGTH / 2.0 + 6.0) 
     && (fabs(v.getY()) < GOAL_AREA_WIDTH / 2.0))
    return true;

  return false;
}






