/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// clearballperception.cpp Thu Jul 19 2001
// Description:
//////////////////////////////////////////////////////////////////////

#include "clearballperception.h"

ClearBallPerception::ClearBallPerception(const WorldModel& _wmWorld) : Perception(_wmWorld)
{
}

ClearBallPerception::~ClearBallPerception()
{
}

bool ClearBallPerception::atBall() const
{
	WorldBall wbBall = wmWorld.getWorldBall();
	ClientWorldPlayer cwpClientPlayer = wmWorld.getClientPlayer();
	
	return (dist(cwpClientPlayer, wbBall) < KICKRANGE);
}

bool ClearBallPerception::inDefensiveThird(const WorldObject& _woObj) const
{
	//gets the absolute position of the WMO
	Vector2d vAbsPos = _woObj.getAbsPosition();
	
	//finds if the X coord is less than -1/6 Length because less than that
	//is the defensive third of the field
	if ( vAbsPos.getX() < -(FIELD_LENGTH / 6.0) )
		return true;
	else return false;
}

bool ClearBallPerception::inDefensiveHalf(const WorldObject& _woObj) const
{
	//gets the absolute position of the WMO
	Vector2d vAbsPos = _woObj.getAbsPosition();
	
	//finds if the X coord is less than -1/6 Length because less than that
	//is the defensive third of the field
	if ( vAbsPos.getX() < 0 )
		return true;
	else return false;
}


bool ClearBallPerception::onDefense() const
{
	ClientWorldPlayer cwpClientPlayer = wmWorld.getClientPlayer();
	
//	return inDefensiveThird(cwpClientPlayer);
	return inDefensiveHalf(cwpClientPlayer);
}

Angle ClearBallPerception::angleToKick() const
{
  ClientWorldPlayer cwpClientPlayer = wmWorld.getClientPlayer();
  Vector2d vPlayerPos = cwpClientPlayer.getAbsPosition();
  
  Angle aKick;
  
  if (vPlayerPos.getX () < -20.0
      && ((fabs (vPlayerPos.getY() < 20.0))))
    {
      if (vPlayerPos.getY() < 0)
	{
	  aKick = relPos(FLAGPOS(FLAG_TOP_LEFT_10)).getAngle();
	}
      else
	{
	  aKick = relPos(FLAGPOS(FLAG_BOTTOM_LEFT_10)).getAngle();
	}
    }
  else
    {
      if (vPlayerPos.getY() < 0)
	{
	  aKick = relPos(FLAGPOS(FLAG_RIGHT_TOP_30)).getAngle();
	}
      else
	{
	  aKick = relPos(FLAGPOS(FLAG_RIGHT_BOTTOM_30)).getAngle();
	}
    }
  
  return aKick;
}

