
/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// collisionbehavior.cpp Tue Jul 10 2001
// Description:
//////////////////////////////////////////////////////////////////////


#include "collisionbehavior.h"

const float CLOSE_THRESHOLD = 15.0;

CollisionBehavior::CollisionBehavior(const WorldModel& _wmWorld) : perCollision(_wmWorld)
{
}

CollisionBehavior::~CollisionBehavior()
{
}

bool CollisionBehavior::isUseful()
{
    if(!perCollision.isActiveMode()) return false;

    if(perCollision.getClosestOpponent().getLength() > CLOSE_THRESHOLD) return false;

    return true;
}


void CollisionBehavior::evaluate(RecommendationList& recList)
{
	Recommendation recFinal;
    recFinal.szBehaviorName = "collision";	
	
    // Figure out possession
    if(perCollision.getPossession() == PS_THEIRS) // Defense!
    {
        return;
    }
	
	RunAction raRun;
	
	// first, get the player we might hit
	Vector2d vNearest = perCollision.getClosestOpponent();
	Angle aObject = vNearest.getAngle();
	
	// If I have the ball, kick it in a direction that won't hit an opponent (dribble, not pass)
	// TODO: Think about calculating an angle instead of using 90 degrees
	if(perCollision.atBall() && (vNearest.getLength() < 5.0 )) // magic #
	{
	    DribbleAction daDrib;
       	if(aObject > 0) // Object is to the right
            daDrib.setDirection(aObject - Angle(M_PI/2));	
       	else // Object to the left
            daDrib.setDirection(aObject + Angle(M_PI/2));

        recFinal.addAction(daDrib);
        // scale the grade
        recFinal.setGrade( 90 ); // **** Combine with gotoGoal()'s dribbleaction
        recList.insert(recFinal);
        return;
	}
	
	else if(perCollision.atBall()) // have ball and not being guarded
	{
	    recFinal.setGrade( 20 );
	    recList.insert(recFinal);
	    return;
	}
	
	// remaining case: offender, get away from opponent (try to get open!)	
	
	if(aObject > 0) // Object is to the right
        raRun.setDirection(aObject - Angle(M_PI/2));	
   	else // Object to the left
        raRun.setDirection(aObject + Angle(M_PI/2));

    raRun.setIntensity( 75 );	// TODO: scale this
	recFinal.addAction(raRun);
	
	// scale the grade 	(the length should never be more than CLOSE_THRESHOLD
	float fGrade =  1 / (vNearest.getLength()+1) * 60;
	if(fGrade > 90) fGrade = 90; // Never want this to take complete control
	
	
	recFinal.setGrade( fGrade ); // it is important to avoid defenders when a teammate has the ball

	recList.insert(recFinal);
}
