/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// CornerPushbehavior.cpp Fri Aug 3 2001
// Description:
//////////////////////////////////////////////////////////////////////

#include "cornerpushbehavior.h"
#include "runaction.h"

CornerPushBehavior::CornerPushBehavior(const WorldModel &_wmWorld): perCornerPush (_wmWorld)
{
}

CornerPushBehavior::~CornerPushBehavior()
{
}

bool CornerPushBehavior::isUseful()
{
  if (perCornerPush.isActiveMode() 
      && (perCornerPush.getPossession() != PS_THEIRS)
      && perCornerPush.isScoringOpportunity ())
    {
      return true;
    }

  return false;
}

void CornerPushBehavior::evaluate (RecommendationList &recList)
{
  TRACE("CornerPushBehavior::evaluate() got called!");
  Recommendation recFinal;
  recFinal.szBehaviorName = "CornerPush";
  
  if (!perCornerPush.amClose())
    {
      RunAction raRun;
      Vector2d target;

      if (perCornerPush.inTopHalf ())
	{
	  target = Vector2d (0.5 * FIELD_LENGTH - GOAL_AREA_LENGTH - 3.0,
			     -.45 * GOAL_AREA_WIDTH);
	}
      else
	{
	  target = Vector2d (0.5 * FIELD_LENGTH - GOAL_AREA_LENGTH - 3.0,
			     .45 * GOAL_AREA_WIDTH);
	}

      raRun.setDirection (perCornerPush.relPos (target).getAngle());
      raRun.setIntensity(80);
      recFinal.addAction(raRun);	
      recFinal.addAction(raRun);
      recFinal.setGrade(80);
    }
  else
    {
      RunAction raRun;
      raRun.setDirection(0);
      raRun.setIntensity(0);
      recFinal.addAction(raRun);	
      recFinal.setGrade(10);
    }
  
  recList.insert(recFinal);
}
