/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// freekickperception.cpp Tue Jul 24 2001
// Description:
//////////////////////////////////////////////////////////////////////

#include "freekickperception.h"

FreeKickPerception::FreeKickPerception(const WorldModel& _wmWorld) : Perception(_wmWorld)
{
}

FreeKickPerception::~FreeKickPerception()
{
}

bool FreeKickPerception::isActiveMode() const
{
    play_mode pmTemp = wmWorld.getPlayMode();
    if( (pmTemp == PLAY_MODE_KICK_IN_US)
      ) return true;

    return false;
}

bool FreeKickPerception::atBall() const
{
	ClientWorldPlayer cwpPlayer = wmWorld.getClientPlayer();
	
	WorldBall wbBall = wmWorld.getWorldBall();
	
	return (dist(cwpPlayer, wbBall) < KICKRANGE);
}


Angle FreeKickPerception::angleToBackOfBall() const
{
	WorldBall wbBall = wmWorld.getWorldBall();
	double dX = wbBall.getAbsPosition().getX() - wbBall.getSize() / 2.0;
	Vector2d vBackOfBall(dX, wbBall.getAbsPosition().getY());
	
	return relPos(vBackOfBall).getAngle();
}

bool FreeKickPerception::isBehindBall() const
{
	WorldBall wbBall = wmWorld.getWorldBall();
	ClientWorldPlayer cwpClientPlayer = wmWorld.getClientPlayer();
	
	if (cwpClientPlayer.getAbsPosition().getX() > wbBall.getAbsPosition().getX())
		return false;
	
	return true;
}

bool FreeKickPerception::isNoOneBehindBall() const
{
	WorldBall wbBall = wmWorld.getWorldBall();
	double dX = wbBall.getAbsPosition().getX();
	
	WorldPlayer wpPlayer;
	
	for(int i = 1; i <= MAX_PLAYER; i++)
	{
		wpPlayer = wmWorld.getOurPlayer(i);
		if (wpPlayer.getAbsPosition().getX() < dX)
			return true;
	}

	return false;
}

bool FreeKickPerception::isCloseToBall() const
{
	WorldPlayer wpPlayer;
	ClientWorldPlayer cwpClientPlayer= wmWorld.getClientPlayer();
	int iClientPlayerNum = cwpClientPlayer.getNumber();
	WorldBall wbBall = wmWorld.getWorldBall();
	double dDist;
	double dShortestDist = INFINITY;
	double dSecondShortestDist = INFINITY;
	int iClosestJerseyNum = UNKNOWN;
	int iSecondClosestNum = UNKNOWN;
	
	for(int i = 1; i <= MAX_PLAYER; i++)
	{
		wpPlayer = wmWorld.getOurPlayer(i);
		if (wpPlayer.isKnown())
		{
    		dDist = dist(wpPlayer, wbBall);
    		if (dDist < dSecondShortestDist)
    		{
    			if (dDist < dShortestDist)
    			{
  	  			dSecondShortestDist = dShortestDist;
  	  			dShortestDist = dDist;
    				iSecondClosestNum = iClosestJerseyNum;
    				iClosestJerseyNum = wpPlayer.getNumber();		
    			}
    			else
    			{
    				dSecondShortestDist = dDist;
    				iSecondClosestNum = wpPlayer.getNumber();
    			}
    		}
    	}
  	}
	
	if ((iSecondClosestNum == iClientPlayerNum) || (iClosestJerseyNum == iClientPlayerNum))
		return true;

	return false;	
}

Angle FreeKickPerception::angleToKick() const
{
  ClientWorldPlayer cwpClientPlayer = wmWorld.getClientPlayer();
  Vector2d vPlayerPos = cwpClientPlayer.getAbsPosition();
  
  Angle aKick;
  
  if (vPlayerPos.getY() < 0)
    {
      aKick = relPos(FLAGPOS(FLAG_RIGHT_TOP_30)).getAngle();
    }
  else
    {
      aKick = relPos(FLAGPOS(FLAG_RIGHT_BOTTOM_30)).getAngle();
    }
  
  return aKick;
}

int FreeKickPerception::getTime() const
{
  return wmWorld.getTime();
}


Angle FreeKickPerception::getRunAngle() const
{
  Vector2d vBallPos = wmWorld.getWorldBall().getAbsPosition();
  vBallPos.setX(vBallPos.getX() - 5);

  return relPos(vBallPos).getAngle();
}


bool FreeKickPerception::inPosition() const
{
  Vector2d vBallPos = wmWorld.getWorldBall().getAbsPosition();
  Vector2d vPlayerPos = wmWorld.getClientPlayer().getAbsPosition();

  if(vPlayerPos.getX() < vBallPos.getX() && fabs(vBallPos.getY() - vPlayerPos.getY()) < 1)
    return true;

  return false;  

}



