/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// perception.cpp Fri Jun 8 2001
// Description:
//////////////////////////////////////////////////////////////////////

#include "perception.h"
#include "clientworldplayer.h"

Perception::Perception(const WorldModel&  _wmWorld): wmWorld(_wmWorld)
{
}

Perception::~Perception()
{
}

/*
Vector2d Perception::relPos(const WorldPlayer &_wpPlayer)
{
  ClientWorldPlayer cwpClientPlayer = wmWorld.getClientPlayer();
  Vector2d vRelPos;

  // compute the vector from the client player to the world player
  vRelPos = _wpPlayer.getAbsPosition() - cwpClientPlayer.getAbsPosition();
  // rotate the vector to be relative to the player's face dir
  vRelPos.setRadAng(vRelPos.getLength(), vRelPos.getAngle() - cwpClientPlayer.getFaceDir());
  return vRelPos;
}
*/

Vector2d Perception::relPos(const WorldObject &_woObject) const
{
  ClientWorldPlayer cwpClientPlayer = wmWorld.getClientPlayer();
  Vector2d vRelPos;

  // compute the vector from the worldobject to the world player
  vRelPos = _woObject.getAbsPosition() - cwpClientPlayer.getAbsPosition();
  // rotate the vector to be relative to the player's face dir
  vRelPos.setRadAng(vRelPos.getLength(), vRelPos.getAngle() - cwpClientPlayer.getBodyDir());
  return vRelPos;
}

Vector2d Perception::relPos(const Vector2d &_vAbsPos) const
{
	ClientWorldPlayer cwpClientPlayer = wmWorld.getClientPlayer();
  Vector2d vRelPos;

  // compute the vector from the worldobject to the world player
  vRelPos = _vAbsPos - cwpClientPlayer.getAbsPosition();
  // rotate the vector to be relative to the player's face dir
  vRelPos.setRadAng(vRelPos.getLength(), vRelPos.getAngle() - cwpClientPlayer.getBodyDir());
  return vRelPos;
}

Angle Perception::getMyFaceDir()
{
	ClientWorldPlayer cwpClientPlayer = wmWorld.getClientPlayer();
	
	return cwpClientPlayer.getFaceDir();
}

int Perception::getTime() const
{
    return wmWorld.getTime();
}

bool Perception::correctViewMode(const view_width vw, const view_quality vq) const
{
    ClientWorldPlayer cwpPlayer = wmWorld.getClientPlayer();

    return (cwpPlayer.getViewWidth() == vw) && (cwpPlayer.getViewQuality() == vq);
}

Angle Perception::getTurnMoment(const Angle &aDirection) const
{
    ClientWorldPlayer cwpPlayer = wmWorld.getClientPlayer();
	Vector2d vVelocity = cwpPlayer.getVelocity();
	
	double dMoment = (1 + INERTIA_MOMENT * vVelocity.getLength()) * (double)aDirection;
	
	if(dMoment > M_PI)
	{
		return PI;
	}
	
	if(dMoment < -M_PI)
	{
		return -PI;
	}
	
	return Angle(dMoment);
}

possession_state Perception::getPossession() const
{
	if (wmWorld.getWorldBall().getAge() > 5)
		return PS_UNKNOWN;
		
	double vOurDist (INFINITY);
	double vOtherDist(INFINITY);
	double vTempDist;

	//get shortest distance from our team to ball
	for (int i = 1; i <= MAX_PLAYER; ++i)
	{
		if (wmWorld.getOurPlayer(i).isYoung())
		{
			vTempDist = dist( wmWorld.getOurPlayer(i), wmWorld.getWorldBall() );
			if (vTempDist < vOurDist)
				vOurDist = vTempDist;
		}
	} 		

	//get shortest distance from other team to ball
	for (int j = 1; j <= MAX_PLAYER; ++j)
	{
		if (wmWorld.getOtherPlayer(j).isYoung())
		{
			vTempDist = dist( wmWorld.getOtherPlayer(j), wmWorld.getWorldBall() );
			if (vTempDist < vOtherDist)
				vOtherDist = vTempDist;
		}
	} 		
	
	//compare distances to return closest possession, if nobody close then return free
	if (vOurDist < vOtherDist)
	{
		if (vOurDist < 2.0 * KICKRANGE)
			return PS_OURS;
	}
	else if (vOtherDist < 2.0 * KICKRANGE)
			return PS_THEIRS;

	return PS_FREE;
	
}

double Perception::dist (const WorldObject& _woObj1, const WorldObject& _woObj2) const
{
  //subtracts the relative position of two objects, and calls the
	//norm function to find the distance of the vector
	return  norm ( _woObj1.getAbsPosition() - _woObj2.getAbsPosition() );
}

bool Perception::amIClosestToBall (int nth) const
{
  WorldPlayer wpPlayer;
  int iClientPlayerNum = (wmWorld.getClientPlayer()).getNumber();
  WorldBall wbBall = wmWorld.getWorldBall();
  double dDist = INFINITY;
  double dShortestDist = INFINITY;
  double dSecondShortestDist = INFINITY;
  int iClosestJerseyNum = -1;
  int iSecondClosestNum = -1;

  if (wbBall.getAbsPosition ().getX () > 0.0)
    {
      nth = 1;
    }

  for (int i = 1; i <= MAX_PLAYER; i++)
    {
      wpPlayer = wmWorld.getOurPlayer(i);

      if (wpPlayer.isKnown())
	{
	  dDist = dist(wpPlayer, wbBall);

	  if (dDist < (dSecondShortestDist - 2.0))
	    {
	      if (dDist < (dShortestDist - 2.0))
		{
		  dSecondShortestDist = dShortestDist;
		  dShortestDist = dDist;
		  iClosestJerseyNum = wpPlayer.getNumber();
		  iSecondClosestNum = iClosestJerseyNum;
		}
	      else
		{
		  dSecondShortestDist = dDist;
		  iSecondClosestNum = wpPlayer.getNumber ();
		}
	    }
  	}
    }
  
  if (nth == 2)
    {
      return ((iSecondClosestNum == iClientPlayerNum) 
	      || (iClosestJerseyNum == iClientPlayerNum));
    }
  else if (nth == 1)
    {
      return (iClosestJerseyNum == iClientPlayerNum);
    }
  else
    {
      ENSURE (false);
    }
  
  return false;
}

double Perception::getStamina() const
{
  return wmWorld.getClientPlayer().getStamina();
}

bool Perception::ballTooOld (int age) const
{
  WorldBall wbBall = wmWorld.getWorldBall();
  return (wbBall.getAge() > age);
}

bool Perception::atBall() const
{
  ClientWorldPlayer cwpClientPlayer = wmWorld.getClientPlayer();
  WorldBall wbBall = wmWorld.getWorldBall();
  
  return (dist(cwpClientPlayer, wbBall) < KICKRANGE);
}

