/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// recommendationlist.cpp Thu Jul 5 2001
// Description:
//////////////////////////////////////////////////////////////////////

#include "recommendationlist.h"
#include <iostream.h>

using namespace std;


RecommendationList::RecommendationList()
{
    clear();
}

RecommendationList::RecommendationList(const RecommendationList& _recOther)
{
	if(this == & _recOther) return;
	clear();	
	
	if(_recOther.vRecPtrs.empty()) return;	
	
	vector<Recommendation*>::const_iterator i;
	
	i = _recOther.vRecPtrs.begin();
	for(; i != _recOther.vRecPtrs.end(); i++)
	{
	    ENSURE( (*i) != NULL );
	    insert( (*i)->copy() );	
	}
	
}

RecommendationList::~RecommendationList()
{
    clear();
}

Recommendation RecommendationList::compose()
{
    Recommendation *recfinal = NULL;

	Recommendation *rec1 = NULL;
	Recommendation *rec2 = NULL;
	
    if(size() == 0) return Recommendation();
    else if(size() == 1)
    {
        recfinal = vRecPtrs.back();
        vRecPtrs.pop_back();
    }
    else if(size() == 2)
    {
        Recommendation *rec1 = vRecPtrs.back(); // should be biggest (grade)
        vRecPtrs.pop_back();
        Recommendation *rec2 = vRecPtrs.back();
        vRecPtrs.pop_back();
        recfinal = combine(*rec1, *rec2);
        delete rec1;
        delete rec2;
    }
    else    // if we're here, size >= 2
    {
        while(vRecPtrs.size() >= 2)
        {
            rec1 = vRecPtrs.back();
            vRecPtrs.pop_back();
            rec2 = vRecPtrs.back();
            vRecPtrs.pop_back();
            Recommendation* rec3;
            rec3 = combine(*rec1, *rec2);
            delete rec1;
            delete rec2;
            vRecPtrs.push_back(rec3);
        }
        ENSURE(vRecPtrs.size() == 1);
        recfinal = vRecPtrs.back();
        vRecPtrs.pop_back();
    }

    ENSURE(vRecPtrs.empty());

    Recommendation recReturn = *recfinal;
    delete recfinal;
    clear();

/*
    if (vRecPtrs.empty())
		return Recommendation();
	
	Recommendation recReturn = *vRecPtrs[vRecPtrs.size()-1];
	
*/
	return recReturn;
	
}

void RecommendationList::insert(const Recommendation& _rec)
{
	if(_rec.getGrade() == 0) return;
	
	Recommendation* recNew = _rec.copy();
	
	insert(recNew);
}

void RecommendationList::insert(Recommendation* _prec)
{

    if(_prec->getGrade() <= 0)
    {
        delete _prec;
        return;
    }

    // empty list
    if(vRecPtrs.empty())
    {
        vRecPtrs.push_back(_prec);
        return;
    }

    // middle of list
    vector<Recommendation*>::iterator i;
    for(i=vRecPtrs.begin(); i != vRecPtrs.end(); ++i)
    {
        if((*i != NULL) && (**i > *_prec)) // if i is greater than prec, put prec before i
        {
            vRecPtrs.insert(i, _prec);
            return;
        }
    }

    // end of list (prec is greatest)
    vRecPtrs.push_back(_prec);
}

ofstream& RecommendationList::ofstream_insert(ofstream& _fout) const
{

    vector<Recommendation*>::const_iterator i;
    for(i=vRecPtrs.begin(); i != vRecPtrs.end(); ++i)
    {
        _fout << **i;
    }

    return _fout;
}

ostream& RecommendationList::ostream_insert(ostream& _out) const
{
    vector<Recommendation*>::const_iterator i;
    for(i=vRecPtrs.begin(); i != vRecPtrs.end(); ++i)
    {
        _out << **i;
    }

    return _out;
}

void RecommendationList::clear()
{
    Recommendation *rec;
	while(!vRecPtrs.empty())
	{
        rec = vRecPtrs.back();
	    vRecPtrs.pop_back();
	    delete rec;
	}		
}

unsigned int RecommendationList::size() const
{
    return vRecPtrs.size();
}


/*
Recommendation* RecommendationList::operator[](unsigned int i) const
{
	ENSURE((i < vRecPtrs.size()) && (i >= 0));
		//return NULL;
		
	return vRecPtrs[i];
}
*/



ofstream& operator<<(ofstream& _fout, const RecommendationList& _rclList)
{
    return _rclList.ofstream_insert(_fout);
}

ostream& operator<<(ostream& _out, const RecommendationList& _rclList)
{
    return _rclList.ostream_insert(_out);
}
