/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// baseworldmodel.h Thu Jun 28 2001
// Description:
//////////////////////////////////////////////////////////////////////

#ifndef BASEWORLDMODEL_H
#define BASEWORLDMODEL_H

#include <iostream.h>
#include <fstream.h>
#include <string.h>
#include <stdlib.h>

#include "constants.h"
#include "angle.h"
#include "vector2d.h"
#include "debug.h"
#include "types.h"
#include "worldball.h"
#include "worldobject.h"
#include "worldplayer.h"
#include "sensor.h"

class BaseWorldModel
{
    friend class LogCapsule;

public: 
	BaseWorldModel();
	virtual ~BaseWorldModel();
	
	play_mode getPlayMode() const;
	playing_side getPlayingSide() const;
	int getOurScore() const;
	int getOtherScore() const;
	int getTime() const;
	char* getTeamName() const;
	
	WorldObject getOurGoal() const;
	WorldObject getOtherGoal() const;
	WorldPlayer getOurGoalie() const;
	WorldPlayer getOtherGoalie() const;
	
	WorldBall getWorldBall() const;
	WorldPlayer getOurPlayer(int _iPlayerNum) const;
	WorldPlayer getOtherPlayer(int _iPlayerNum) const;
	
	bool inSecondHalf() const;
	bool isTimeRunning() const;
	
	void setOurGoalie (WorldPlayer* _pwpGoalie);
    void setOtherGoalie (WorldPlayer* _pwpGoalie);	
	//void setOurGoal(goal_type _gtWhichGoal);
	//void setOtherGoal(goal_type _gtWhichGoal);
	void setPlayMode(play_mode _pmPlayMode);
	void setOurScore(int _iOurScore);
	void setOtherScore(int _iOtherScore);
	virtual void setTime(int _iTime);
	void setTeamName(char* _pcTeamName);
	
	// also used by AudioData
	void updateWorldPlayer(const WorldPlayer& _wpPlayer); // Replayer facility

		
	virtual ofstream& ofstream_insert(ofstream& _out);
    virtual ifstream& ifstream_extract(ifstream& _in);




protected:	
    virtual void age();         // Increase the age of all known objects (ball and players)

    WorldBall wbBall;										
    WorldPlayer* pwpOurTeam[MAX_PLAYER];		// array of players on our team indexed by jersey number
    WorldPlayer* pwpOtherTeam[MAX_PLAYER]; 	// array of players on other team indexed by jersey number

    WorldPlayer* pwpOurGoalie;						// pointer to our goalie for quick reference
    WorldPlayer* pwpOtherGoalie;					// pointer to other goalie for quick reference

	WorldObject woOurGoal;								// Our goal
    WorldObject woOtherGoal;							// Other Goal

    play_mode pmPlayMode;									// mode of play (ie before kick off, play on, goal kick us, etc.)
    int iOurScore;												// our score
    int iOtherScore;											// opponents score
    int iTime; 														// Current time - don't forget to check for dropped server data
    char *pcTeamName;                     // Name of our team, passed in from command line
//    static int count;	
};

#endif
